@@ -17,6 +17,9 @@
 #define JOOLNL_FAMILY "Jool"
 #define JOOLNL_MULTICAST_GRP_NAME "joold"
 
+#define JOOLNL_HDR_MAGIC "jool"
+#define JOOLNL_HDR_MAGIC_LEN 4
+
 enum joolnl_operation {
 	JNLOP_INSTANCE_FOREACH,
 	JNLOP_INSTANCE_ADD,
@@ -290,6 +293,8 @@ typedef __u8 joolnlhdr_flags; /** See JOOLNLHDR_FLAGS_* above. */
  * nlmsghdr, genlmsghdr)
  */
 struct joolnlhdr {
+	/** Always "jool". (Without terminating character.) */
+	char magic[JOOLNL_HDR_MAGIC_LEN];
 	/** Jool's version. */
 	__be32 version;
 
