@@ -8,7 +8,8 @@
 char *get_iname(struct genl_info *info);
 struct joolnlhdr *get_jool_hdr(struct genl_info *info);
 
-int request_handle_start(struct genl_info *info, xlator_type xt, struct xlator *jool);
+int request_handle_start(struct genl_info *info, xlator_type xt,
+		struct xlator *jool, bool require_net_admin);
 void request_handle_end(struct xlator *jool);
 
 #endif /* SRC_MOD_COMMON_NL_COMMON_H_ */
