@@ -275,7 +275,7 @@ static int add_to_root(struct rtrie *trie, struct rtrie_node *new)
 }
 
 static int add_full_collision(struct rtrie *trie, struct rtrie_node *parent,
-		struct rtrie_node *new)
+		struct rtrie_node *new, bool synchronize)
 {
 	/*
 	 * We're adding new to
@@ -342,7 +342,8 @@ static int add_full_collision(struct rtrie *trie, struct rtrie_node *parent,
 
 	rcu_assign_pointer(parent->left, NULL);
 	rcu_assign_pointer(parent->right, NULL);
-	synchronize_rcu_bh();
+	if (synchronize)
+		synchronize_rcu_bh();
 	rcu_assign_pointer(parent->left, smallest_prefix);
 	rcu_assign_pointer(parent->right, inode);
 
@@ -356,7 +357,8 @@ static int add_full_collision(struct rtrie *trie, struct rtrie_node *parent,
 	return 0;
 }
 
-int rtrie_add(struct rtrie *trie, void *value, size_t key_offset, __u8 key_len)
+int rtrie_add(struct rtrie *trie, void *value, size_t key_offset, __u8 key_len,
+		bool synchronize)
 {
 	struct rtrie_node *new;
 	struct rtrie_node *parent;
@@ -405,7 +407,8 @@ int rtrie_add(struct rtrie *trie, void *value, size_t key_offset, __u8 key_len)
 		RCU_INIT_POINTER(new->right, right);
 		rcu_assign_pointer(parent->left, NULL);
 		rcu_assign_pointer(parent->right, NULL);
-		synchronize_rcu_bh();
+		if (synchronize)
+			synchronize_rcu_bh();
 		rcu_assign_pointer(parent->right, new);
 
 		left->parent = new;
@@ -429,7 +432,7 @@ int rtrie_add(struct rtrie *trie, void *value, size_t key_offset, __u8 key_len)
 		goto simple_success;
 	}
 
-	return add_full_collision(trie, parent, new);
+	return add_full_collision(trie, parent, new, synchronize);
 
 simple_success:
 	new->parent = parent;
@@ -479,7 +482,7 @@ bool rtrie_is_empty(struct rtrie *trie)
 	return result;
 }
 
-int rtrie_rm(struct rtrie *trie, struct rtrie_key *key)
+int rtrie_rm(struct rtrie *trie, struct rtrie_key *key, bool synchronize)
 {
 	struct rtrie_node *node;
 	struct rtrie_node *new;
@@ -501,7 +504,8 @@ int rtrie_rm(struct rtrie *trie, struct rtrie_key *key)
 		parent_ptr = get_parent_ptr(trie, node);
 
 		rcu_assign_pointer(*parent_ptr, new);
-		synchronize_rcu_bh();
+		if (synchronize)
+			synchronize_rcu_bh();
 
 		deref_updater(trie, new->left)->parent = new;
 		deref_updater(trie, new->right)->parent = new;
@@ -522,7 +526,8 @@ int rtrie_rm(struct rtrie *trie, struct rtrie_key *key)
 
 		if (node->left) {
 			rcu_assign_pointer(*parent_ptr, node->left);
-			synchronize_rcu_bh();
+			if (synchronize)
+				synchronize_rcu_bh();
 			deref_updater(trie, node->left)->parent = parent;
 			list_del(&node->list_hook);
 			__wkfree("Rtrie node", node);
@@ -531,7 +536,8 @@ int rtrie_rm(struct rtrie *trie, struct rtrie_key *key)
 
 		if (node->right) {
 			rcu_assign_pointer(*parent_ptr, node->right);
-			synchronize_rcu_bh();
+			if (synchronize)
+				synchronize_rcu_bh();
 			deref_updater(trie, node->right)->parent = parent;
 			list_del(&node->list_hook);
 			__wkfree("Rtrie node", node);
@@ -539,7 +545,8 @@ int rtrie_rm(struct rtrie *trie, struct rtrie_key *key)
 		}
 
 		rcu_assign_pointer(*parent_ptr, NULL);
-		synchronize_rcu_bh();
+		if (synchronize)
+			synchronize_rcu_bh();
 		list_del(&node->list_hook);
 		__wkfree("Rtrie node", node);
 
