@@ -32,6 +32,8 @@ static char const *banner = "\n"
 	";   |.'      `--''                      \\   \\####/      '  ,/   \n"
 	"'---'                                    `---`--`       '--'    \n";
 
+#ifndef XTABLES_DISABLED
+
 static int iptables_error;
 
 /** iptables module registration object */
@@ -55,6 +57,8 @@ static struct xt_target targets[] = {
 	},
 };
 
+#endif /* !XTABLES_DISABLED */
+
 static void flush_net(struct net *ns)
 {
 	jool_xlator_flush_net(ns, XT_NAT64);
@@ -88,24 +92,29 @@ static int __init nat64_init(void)
 
 	pr_debug("%s", banner);
 	pr_debug("Inserting NAT64 Jool...\n");
+
 	/* Careful with the order */
 
 	error = register_pernet_subsys(&joolns_ops);
 	if (error)
 		return error;
 
+#ifndef XTABLES_DISABLED
 	iptables_error = xt_register_targets(targets, ARRAY_SIZE(targets));
 	if (iptables_error) {
 		log_warn("Error code %d while trying to register the iptables targets.\n"
 				"iptables SIIT Jool will not be available.",
 				iptables_error);
 	}
+#endif
 
 	/* NAT64 instances can now function properly; unlock them. */
 	error = jool_nat64_get(defrag_enable);
 	if (error) {
+#ifndef XTABLES_DISABLED
 		if (!iptables_error)
 			xt_unregister_targets(targets, ARRAY_SIZE(targets));
+#endif
 		unregister_pernet_subsys(&joolns_ops);
 		return error;
 	}
@@ -117,8 +126,10 @@ static int __init nat64_init(void)
 static void __exit nat64_exit(void)
 {
 	jool_nat64_put();
+#ifndef XTABLES_DISABLED
 	if (!iptables_error)
 		xt_unregister_targets(targets, ARRAY_SIZE(targets));
+#endif
 	unregister_pernet_subsys(&joolns_ops);
 	pr_info("NAT64 Jool v" JOOL_VERSION_STR " module removed.\n");
 }
