@@ -1,4 +1,4 @@
-#include "common/config.h"
+#include "common/iptables.h"
 
 #define IPTABLES_MODULE_NAME IPTABLES_NAT64_MODULE_NAME
 #define IPTABLES_MODULE_TYPE XT_NAT64
